import json
from typing import Union, Dict, Any, List

from csp_cbc_auth.cbc.session import CBCSession
from psc_auth_integration.entities.auth_user_session import AuthUserSession

from cds_configuration_service.api_clients.base_api_class import ConfigurationServiceBaseClass


class GlobalConfigurationServiceV1Api(ConfigurationServiceBaseClass):
    """
    Global Configuration Service V1 API wrapper
    """

    def __init__(self, session: Union[AuthUserSession, CBCSession]):
        super().__init__(session)
        self.global_base_url = f'{self.configuration_service_base_url}/pacing_config/internal/v1/global/profiles'

    def get_all_profiles(self):
        """
        Retrieve all global profiles.
        """
        url = self.global_base_url
        return self.get(url)

    def add_profile(self, profile_data: Dict[str, Any]):
        """
        Add a new global profile.

        :param profile_data: A dictionary of profile data.
            Example structure of the `profile_data`:
            {
                "content_identifiers": ["identifier_1", "identifier_2", ...],  # list of content identifiers
                "budget": [
                    # Multiple budget data entries in a 7 x 24 matrix
                    [50.0, 50.0, 1.0, ..., 1.0, 1.0],
                    [50.0, 50.0, 1.0, ..., 1.0, 1.0],
                    ...
                ]
            }
        """
        url = self.global_base_url
        return self.post(url, json=profile_data)

    def update_profile(self, profile_key: str, budget_data: List[List[float]]):
        """
        Update a global profile's budget.

        :param profile_key: A string that uniquely identifies the global profile to be updated.
            Example structure of the `profile_key`: "VMWARE_CBC:::OSQUERY"
        :param budget_data: A list of lists containing float values that represent the budget allocations.
            It should be a 7x24 matrix where each inner list contains 24 float values for each hour of the day,
            and the outer list contains 7 of these inner lists, representing a week. Each float value represents
            the specific budget setting for that hour.

            Example structure of the `budget_data`:
            [
                [100.0, 50.0, 1.0, ..., 1.0, 1.0],
                [50.0, 50.0, 1.0, ..., 1.0, 1.0],
                ...
                [50.0, 50.0, 1.0, ..., 1.0, 1.0]
            ]

        """
        url = f'{self.global_base_url}/{profile_key}'
        data = json.dumps(budget_data)
        return self.patch(url, data=data)

    def delete_profile(self, profile_key: str):
        """
        Delete a global profile.

        :param profile_key: A string that uniquely identifies the global profile to be updated.
            Example structure of the `profile_key`: "VMWARE_CBC:::OSQUERY"
        """
        url = f'{self.global_base_url}/{profile_key}'
        return self.delete(url)
