import pytest

from cds_configuration_service.api_clients.global_configuration_service_v1 import GlobalConfigurationServiceV1Api
from cds_configuration_service.tests.plugins.ownership_plugin import SGUID
from cds_configuration_service.tests.utils.auth import CDSSession


@pytest.mark.sguid(SGUID.psc_content_mgmt_app_cds_configuration_service)
class TestGlobalProfileManagement:
    TARGET_PROFILE_KEY = 'VMWARE_CBC:::TEST'

    @staticmethod
    def create_new_profile_data():
        """Returns the standard profile data for testing."""
        return {
            'content_identifiers': ['TEST'],
            'budget': [
                [50.0, 50.0, 1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 100.0, 100.0, 10.0, 1.0, 100.0,
                 100.0, 1.0, 1.0, 10.0, 1.0, 1.0],
                [50.0, 50.0, 1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 100.0, 100.0, 100.0, 1.0, 100.0,
                 100.0, 1.0, 1.0, 10.0, 1.0, 1.0],
                [50.0, 50.0, 1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 100.0, 100.0, 100.0, 1.0, 100.0,
                 100.0, 1.0, 1.0, 10.0, 1.0, 1.0],
                [50.0, 50.0, 1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 100.0, 100.0, 10.0, 1.0, 10.0,
                 100.0, 1.0, 1.0, 10.0, 1.0, 1.0],
                [50.0, 50.0, 1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 100.0, 100.0, 100.0, 1.0, 100.0,
                 100.0, 1.0, 1.0, 10.0, 1.0, 1.0],
                [50.0, 50.0, 1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 100.0, 100.0, 100.0, 1.0, 100.0,
                 100.0, 1.0, 1.0, 10.0, 1.0, 1.0],
                [50.0, 50.0, 1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 100.0, 100.0, 100.0, 1.0, 100.0,
                 100.0, 1.0, 1.0, 10.0, 1.0, 1.0]]
        }

    def test_global_profile_workflow(self, auth_session: CDSSession,
                                     global_configuration_service_v1_api: GlobalConfigurationServiceV1Api):
        """
        Test the workflow for managing a global profile in the configuration service.

        This test performs a complete workflow involved in the global configuration service.
        The steps are:
        1. Verification that the target profile does not exist initially.
        2. Creation of a new global profile with predefined data.
        3. Verification that the profile is created successfully.
        4. Updating a specific field in the profile (e.g., budget).
        5. Verification that the update is successful and data is modified.
        6. Deletion of the newly created profile.
        7. Verification that the profile is successfully deleted.

        Each step is validated by asserting the expected outcomes, such as HTTP status codes
        and the state of the profile data.

        The method also includes a cleanup step to remove the test profile, ensuring a consistent
        state for subsequent tests. This cleanup occurs regardless of the test outcome, making it
        fail-safe against inconsistent test environments.

        Parameters:
        auth_session (CDSSession): A session object representing an authenticated user.
        global_configuration_service_v1_api (GlobalConfigurationServiceV1Api): The API wrapper used
        to interact with the global configuration service.

        The test is considered failed if any of the HTTP responses do not meet the expected
        outcomes or if the profile's state does not reflect the changes made during the operations.
        """
        try:
            # Verifying target profile is None
            new_profile = self.retrieve_profile_by_key(global_configuration_service_v1_api, self.TARGET_PROFILE_KEY)
            assert new_profile is None

            # Creating a new profile
            profile_data = self.create_new_profile_data()
            create_response = global_configuration_service_v1_api.add_profile(profile_data)
            create_response.raise_for_status()

            # Verifying creation
            new_profile = self.retrieve_profile_by_key(global_configuration_service_v1_api, self.TARGET_PROFILE_KEY)
            assert new_profile is not None

            # Updating the profile
            profile_data.get('budget')[0][0] = 100.0
            update_response = global_configuration_service_v1_api.update_profile(self.TARGET_PROFILE_KEY,
                                                                                 profile_data.get('budget'))
            update_response.raise_for_status()

            # Verifying update
            updated_profile = self.retrieve_profile_by_key(global_configuration_service_v1_api, self.TARGET_PROFILE_KEY)
            assert updated_profile['budget'][0][0] == 100.0

            # Deleting the profile
            delete_response = global_configuration_service_v1_api.delete_profile(self.TARGET_PROFILE_KEY)
            delete_response.raise_for_status()

            # Verifying deletion
            deleted_profile = self.retrieve_profile_by_key(global_configuration_service_v1_api, self.TARGET_PROFILE_KEY)
            assert deleted_profile is None
        finally:
            # Cleanup
            cleanup_response = global_configuration_service_v1_api.delete_profile(self.TARGET_PROFILE_KEY)
            assert cleanup_response.status_code in [200, 404]

    def retrieve_profile_by_key(self, api, profile_key):
        """Helper method to retrieve a profile by key from the complete list of profiles."""
        response = api.get_all_profiles()
        assert response.status_code in [200, 404]
        if response.status_code == 200:
            return next((profile for profile in response.json() if profile.get('profile_key') == profile_key), None)
        return None
